#!/bin/bash

# SERIES list
SERIES_LIST=(
	"5010"
	"5010v2"
	"3010"
	"3020"
	"51210"
)

echo "===== set cross build env ======"
export CROSS_COMPILE=arm-linux-gnueabihf-
export ARCH=arm
echo "export CROSS_COMPILE=${CROSS_COMPILE}"
echo "export export ARCH=${ARCH}"

echo "===== start u-boot building ====="
if [ $1 = "all" ] ; then
	echo "make all"
elif [ $1 = "clean" ] ; then
	echo "make clean"

	#clean dev-tool
	make clean -C ./dev-tools/flash_contents/flash_img_create/
	make clean -C ./dev-tools/flash_contents/flash_img_obj_create/
	make clean -C ./dev-tools/flash_contents/flash_img_print/

	#clean dtb files
	rm ./input/alpine_db.dtb

	#clean output files
	echo "rm -rf ./output" && rm -rf ./output

	exit 1
else
	if [ -f ./input/ts${1}_db.dts ] ; then
		echo "make ${1}"
	else
		echo "Fail:"
		echo " too few arguments to function, or too many arguments to function."
		echo " release_test.sh all"
		echo " relese_test.sh <series_name>"
		echo " ie. relese_test.sh 5010"
		exit 1
	fi
fi

# Halt upon errors
set -e

echo "====== build : dev-tool ====="
# Make all tools
make -C ./dev-tools/flash_contents/flash_img_create/
make -C ./dev-tools/flash_contents/flash_img_obj_create/
make -C ./dev-tools/flash_contents/flash_img_print/

echo "====== build : u-boot ======"
export HAL_TOP=$PWD/src/HAL/

# Clone mainline U-Boot
#git clone git://git.denx.de/u-boot.git
#cd u-boot/
#git reset --hard 92fa7f53f1f3f03296f8ffb14bdf1baefab83368

# Apply Annapurna Labs U-Boot patch
#git am ../src/full_patch/0001-AL-Annapurna-Labs-Alpine-DB-Support.patch

# Build U-Boot
make alpine_db_config
make -s -j 9

# Copy U-Boot binary to the image preparation input directory
cp u-boot.bin ./input/

# Compile the Device Tree
for series in "${SERIES_LIST[@]}"
do
	SERIES=`echo ${series}|awk -F "=" '{print $1}'`
	if [ "${1}" != "all" -a "${1}" != "${SERIES}" ] ; then
		continue
	fi
	echo "====== build : ts${SERIES}_db.dtb  ======"
	if [ ! -d "output/${SERIES}" ]; then
		mkdir -p "output/${SERIES}";
	fi
	dtc -o ./input/alpine_db.dtb -O dtb ./input/ts${SERIES}_db.dts
	cp ./input/alpine_db.dtb ./output/${SERIES}/alpine-ts${SERIES}.dtb

	echo "====== build : boot.img(u-boot-ts${SERIES}.buffalo.updated) ======"
	case ${SERIES} in 
	"3010" | "5010" | "51210")
		./dev-tools/flash_contents/scripts/alpine_db_spi_main/create.sh ./input ./output/${SERIES}
		;;
	"3020" | "5010v2" | *)	# dual al-boot @spi(4MiB) for fw4.50-ts3020
		./dev-tools/flash_contents/scripts/alpine_db_spi_main/create_dual_al_boot.sh ./input ./output/${SERIES}
		;;
	esac
	cp ./output/${SERIES}/boot.img ./output/${SERIES}/u-boot-ts${series}.buffalo.updated
done

echo "===== build : COMPLETE !! ====="
echo "output the images to 'output/*'"

# Generate the images
# ./dev-tools/flash_contents/scripts/alpine_db_spi_main/create.sh ./input ./output

# Generate a recovery image
DUMMY_STG2=`mktemp`
echo dummy > $DUMMY_STG2
./recovery/boot_image_create.sh $DUMMY_STG2 ./recovery/stage3.bin 256k ./input/u-boot.bin 0 ./output/boot.img.recovery.tmp
dd if=./output/boot.img.recovery.tmp of=./output/boot.img.recovery.tmp.padded bs=768k conv=sync > /dev/null 2>&1
dd if=./input/alpine_db.dtb of=./output/alpine_db.dtb.padded bs=64k conv=sync > /dev/null 2>&1
cat ./output/boot.img.recovery.tmp.padded ./output/alpine_db.dtb.padded > ./output/boot.img.recovery.tmp.withdt
dd if=./output/boot.img.recovery.tmp.withdt of=./output/boot.img.recovery bs=1 skip=12292 > /dev/null 2>&1
rm ./output/boot.img.recovery.tmp
rm ./output/boot.img.recovery.tmp.withdt
rm ./output/boot.img.recovery.tmp.padded
rm ./output/alpine_db.dtb.padded
rm $DUMMY_STG2

